-- Each row is 200 bytes in size
CREATE TABLE TestingJoinsInt2 (
	ID INT IDENTITY NOT NULL PRIMARY KEY,
	IntForeignKey INT NOT NULL, -- this is the join column that will be differing in type and size.
	SomeArbColumn DATETIME,
	Filler CHAR(180) DEFAULT ' ',
)

-- Each row in this table is 50 bytes in size
CREATE TABLE LookupTableInt2 (
	ID INT NOT NULL PRIMARY KEY,
	SomeArbStatus CHAR(4),
	Filler CHAR(38) DEFAULT ' '
)

CREATE TABLE TestingJoinsString2 (
	ID INT IDENTITY NOT NULL PRIMARY KEY,
	StrForeignKey CHAR(24) NOT NULL, -- this is the join column that will be differing in type and size.
	SomeArbColumn DATETIME,
	Filler CHAR(160) DEFAULT ' ' -- 20 less then for the int, to keep the rows the same size
)

-- Each row in this table is 50 bytes in size
CREATE TABLE LookupTableString2 (
	ID CHAR(24) NOT NULL PRIMARY KEY,
	SomeArbStatus CHAR(4),
	Filler CHAR(18) DEFAULT ' ' -- 20 less then for the int, to keep the rows the same size
)

-- This will take several seconds to run...
-- int tables first
INSERT INTO TestingJoinsInt2 (IntForeignKey, SomeArbColumn)
SELECT NTILE(100000) OVER (Order by object_id, column_id, type), DATEADD(ms,object_id,DATEADD(dd,column_id,create_date)) FROM ( 
	SELECT TOP (2500000) a.object_id, b.column_id, c.type , c.create_date
	FROM master.sys.columns a CROSS JOIN master.sys.columns b CROSS JOIN master.sys.objects c
) sub

-- and populate the lookup table

INSERT INTO LookupTableInt2 (ID, SomeArbStatus) 
SELECT IntForeignKey, REPLICATE(CHAR(65+FLOOR(RAND(IntForeignKey*2511)*10)),4)
FROM
	(SELECT DISTINCT IntForeignKey FROM TestingJoinsInt2) sub
	
-- now string tables

INSERT INTO TestingJoinsString2 (StrForeignKey, SomeArbColumn)
SELECT CAST(NTILE(100000) OVER (Order by object_id, column_id, type) AS CHAR(8)) + CAST(NTILE(100000) OVER (Order by object_id, column_id, type) AS CHAR(8)), 
		DATEADD(ms,object_id,DATEADD(dd,column_id,create_date)) 
FROM ( 
	SELECT TOP (2500000) a.object_id, b.column_id, c.type , c.create_date
	FROM master.sys.columns a CROSS JOIN master.sys.columns b CROSS JOIN master.sys.objects c
) sub

-- and populate the lookup table

INSERT INTO LookupTableString2 (ID, SomeArbStatus) 
SELECT StrForeignKey, REPLICATE(CHAR(65+FLOOR(RAND(CHECKSUM(NEWID()))*10)),4)
FROM
	(SELECT DISTINCT StrForeignKey FROM TestingJoinsString2) sub
GO
-- first test, no indexes on the join columns

SELECT t1.ID, t2.ID, t1.IntForeignKey, t2.SomeArbStatus
	FROM dbo.TestingJoinsInt2 t1 
		INNER JOIN dbo.LookupTableInt2 t2 ON t1.IntForeignKey = t2.ID
GO 10

SELECT t1.ID, t2.ID, t1.StrForeignKey, t2.SomeArbStatus
	FROM dbo.TestingJoinsString2 t1 
		INNER JOIN dbo.LookupTableString2 t2 ON t1.StrForeignKey = t2.ID
GO 10
   
 -- now with indexes on the join columns
 
CREATE NONCLUSTERED INDEX idx_IntJoins ON TestingJoinsInt2 (IntForeignKey)
CREATE NONCLUSTERED INDEX idx_StringJoins ON TestingJoinsString2 (StrForeignKey)
GO

-- cleanup
DROP TABLE TestingJoinsInt2
DROP TABLE LookupTableInt2
DROP TABLE TestingJoinsString2
DROP TABLE LookupTableString2
