-- Each row is 200 bytes in size
CREATE TABLE TestingJoinsInt (
	ID INT IDENTITY NOT NULL PRIMARY KEY,
	IntForeignKey BIGINT NOT NULL, -- this is the join column that will be differing in type.
	SomeArbColumn DATETIME,
	Filler CHAR(180) DEFAULT ' '
)

-- Each row in this table is 50 bytes in size
CREATE TABLE LookupTableInt (
	ID BIGINT NOT NULL PRIMARY KEY,
	SomeArbStatus CHAR(4),
	Filler CHAR(38) DEFAULT ' '
)

CREATE TABLE TestingJoinsString (
	ID INT IDENTITY NOT NULL PRIMARY KEY,
	StrForeignKey CHAR(8) NOT NULL, -- this is the join column that will be differing in type.
	SomeArbColumn DATETIME,
	Filler CHAR(180) DEFAULT ' '
)

-- Each row in this table is 50 bytes in size
CREATE TABLE LookupTableString (
	ID CHAR(8) NOT NULL PRIMARY KEY,
	SomeArbStatus CHAR(4),
	Filler CHAR(38) DEFAULT ' '
)

-- This will take several seconds to run...
-- int tables first
INSERT INTO TestingJoinsInt (IntForeignKey, SomeArbColumn)
SELECT NTILE(100000) OVER (Order by object_id, column_id, type), DATEADD(ms,object_id,DATEADD(dd,column_id,create_date)) FROM ( 
	SELECT TOP (2500000) a.object_id, b.column_id, c.type , c.create_date
	FROM master.sys.columns a CROSS JOIN master.sys.columns b CROSS JOIN master.sys.objects c
) sub

-- and populate the lookup table

INSERT INTO LookupTableInt (ID, SomeArbStatus) 
SELECT IntForeignKey, REPLICATE(CHAR(65+FLOOR(RAND(IntForeignKey*2511)*10)),4)
FROM
	(SELECT DISTINCT IntForeignKey FROM TestingJoinsInt) sub
	
-- now string tables

INSERT INTO TestingJoinsString (StrForeignKey, SomeArbColumn)
SELECT CAST(NTILE(100000) OVER (Order by object_id, column_id, type) AS CHAR(8)), DATEADD(ms,object_id,DATEADD(dd,column_id,create_date)) FROM ( 
	SELECT TOP (2500000) a.object_id, b.column_id, c.type , c.create_date
	FROM master.sys.columns a CROSS JOIN master.sys.columns b CROSS JOIN master.sys.objects c
) sub

-- and populate the lookup table

INSERT INTO LookupTableString (ID, SomeArbStatus) 
SELECT StrForeignKey, REPLICATE(CHAR(65+FLOOR(RAND(StrForeignKey*2511)*10)),4)
FROM
	(SELECT DISTINCT StrForeignKey FROM TestingJoinsString) sub
GO

-- first test, no indexes on the join columns

SELECT t1.ID, t2.ID, t1.IntForeignKey, t2.SomeArbStatus
	FROM dbo.TestingJoinsInt t1 
		INNER JOIN dbo.LookupTableInt t2 ON t1.IntForeignKey = t2.ID
GO 10

SELECT t1.ID, t2.ID, t1.StrForeignKey, t2.SomeArbStatus
	FROM dbo.TestingJoinsString t1 
		INNER JOIN dbo.LookupTableString t2 ON t1.StrForeignKey = t2.ID
GO 10	
   
 -- now with indexes on the join columns
 
CREATE NONCLUSTERED INDEX idx_IntJoins ON TestingJoinsInt (IntForeignKey)
CREATE NONCLUSTERED INDEX idx_StringJoins ON TestingJoinsString (StrForeignKey)
GO

SELECT t1.ID, t2.ID, t1.IntForeignKey, t2.SomeArbStatus
	FROM dbo.TestingJoinsInt t1 
		INNER JOIN dbo.LookupTableInt t2 ON t1.IntForeignKey = t2.ID
GO 10

SELECT t1.ID, t2.ID, t1.StrForeignKey, t2.SomeArbStatus
	FROM dbo.TestingJoinsString t1 
		INNER JOIN dbo.LookupTableString t2 ON t1.StrForeignKey = t2.ID
GO 10	

-- cleanup
DROP TABLE TestingJoinsInt
DROP TABLE LookupTableInt
DROP TABLE TestingJoinsString
DROP TABLE LookupTableString